/*
------------------------------------------------------------------------------
NEW for DevC++     Author: Maurice Luethi, 2012-09-15
                   C-Partner: Tobias Frei, 2012-09-16
                   All rights reserved!!!
                   last modification:      2012-09-21
------------------------------------------------------------------------------
                   This DEMO shows the use of ProfiLab string functions for a
                   strict checking C-compiler like DevC++ (MINGW32). 
                   It may be used as a template for own projects on a strictly
                   non-comercial base (educational institutions are considered
                   to be comercial in this context).
                   All risks are entierly and not excludable by the user.
                   'No risk, no fun!' if you don't agree, don't use at all!!! 
                   No modifications to the supplied files are permitted!
------------------------------------------------------------------------------                   
All necessary includes and Windows calls plus:
DLLEXPORT  function are defined in "dll.hpp".
------------------------------------------------------------------------------
 */
#include "dll.hpp"

// define Pin's INTERNAL (C-code) reference and No.
const unsigned char HEX_IN   = 0;
const unsigned char WORD_IN  = 2;
const unsigned char CLK_IN   = 4;
const unsigned char WORD_OUT = 1;
const unsigned char HEX_OUT  = 3;

const unsigned char DLLIndex = 100;

const unsigned char inputs   = 5;  //five inputs
const unsigned char outputs  = 4;  //four outputs

// ProfiLab string parameter type ( = "array[0..100] of PChar" (for a max. of 100 pins)
// string memory provided by ProfiLab (length of each PChar must not exceed 1000 characters)
typedef unsigned char *PChar;
typedef PChar *StringParam[100];



// Exported function references


// *** needed by PL to configure 'chip' pin's
DLLEXPORT unsigned char _stdcall NumInputs();
DLLEXPORT unsigned char _stdcall NumOutputs();
DLLEXPORT void _stdcall GetInputName(unsigned char Chan, unsigned char *Name);
DLLEXPORT void _stdcall GetOutputName(unsigned char Chan, unsigned char *Name);
// *** end 'chip' definition pin's (names assigned below)

// this is where the ACTION takes place...(main routine of DLL) implemented below
DLLEXPORT void _stdcall CCalculateEx(double *PInput, double *POutput, double *PUser, StringParam PStrings);

// for CONFIGURATION (if at all needed)...   Only display the message "...".
DLLEXPORT void _stdcall CConfigure(double *PUser);



//   ----- PL pin number/name definiton START -----
//
//return number of input channels...
unsigned char _stdcall NumInputs(){ return inputs; }

//return number of output channels...
unsigned char _stdcall NumOutputs() { return outputs; }

//return name for each input...
void _stdcall GetInputName(unsigned char Chan, unsigned char *Name)
{
    //Remark: String inputs must be named with leading '$'
	if (Chan == HEX_IN)  strcpy((char*)Name, "$HEXi");   //Name input 0
	if (Chan == 1)       strcpy((char*)Name, "");        // no name in 1&3
    if (Chan == WORD_IN) strcpy((char*)Name, "Wi");      //Name input 2
    if (Chan == 3)       strcpy((char*)Name, "");
	if (Chan == CLK_IN)  strcpy((char*)Name, "CLK/");    //Name input 4
}

//return name for each output...
void _stdcall GetOutputName(unsigned char Chan, unsigned char *Name)
{
    //Remark: String outputs must be named with leading '$'
    if (Chan == 0)        strcpy((char*)Name, "");       // no name out 0&2
	if (Chan == WORD_OUT) strcpy((char*)Name, "Wo");     //Name output 1
	if (Chan == 2)        strcpy((char*)Name, "");
	if (Chan == HEX_OUT)  strcpy((char*)Name, "$HEXo");  //Name output 3
}
//   ----- PL pin number/name definiton END -----



// points to PUser to store the prev. status of CLK
// variable (double) provided by PL (0 .. 99), 100 = DLL number (don't use!)
const unsigned char CLK_OLD = 0;      // CLK_IN's previous value, needed to check
                                      // for rising flank on CLK (input).
// more of the like HERE, if needed.. 



/* here whould be the INIT function if needed/used (NOT in this case) */
// void _stdcall CSimStart(double *PInput, double *POutput, double *PUser) { ... }
// Before use it must be declared with DLLEXPORT (see above) as the other functions are.



// CCalculateEx allows string parameters ... aaaaaannnndd ACTION! ...
void _stdcall CCalculateEx(double *PInput, double *POutput, double *PUser, StringParam PStrings)
{
   // local variables: declaration (only in this function available)
   int i,w,l;
   char *c, TempStr[BUFSIZ];
   
   // TRIGGER action on rising flank of clock only!
   // Exeption: store actual status of CLK_IN-pin on every call of PL (simula-
   //           tion frequency) to PUser[CLK_OLD]).
   if (PInput[CLK_IN] > 1 && PUser[CLK_OLD] < 1)  // CLK raising? 
   {
      // last try
      sprintf((char*)PChar(PStrings[HEX_OUT]), "%#04x", int (PInput[WORD_IN]));
      
      // INIT var's
      w=0;
      l=strlen((char*)PChar(PStrings[HEX_IN]))-1;
   
      // Hex to Word...
      // Get hex$ characters one by one and weight them...
      for (i=l; i>=0; i--)   // start at the END (of string) amd move towards
      {                      // the beginning (first char in string).
         c = (char*)PChar(PStrings[HEX_IN]);
         if ((c[i]>='0') && (c[i]<='9')) w = w + (c[i]-'0'+ 0) * ( 1 << ((l-i)*4));
         if ((c[i]>='A') && (c[i]<='F')) w = w + (c[i]-'A'+10) * ( 1 << ((l-i)*4));
         if ((c[i]>='a') && (c[i]<='f')) w = w + (c[i]-'a'+10) * ( 1 << ((l-i)*4));
      }
      POutput[WORD_OUT]=w; // return word value
   }
   // Word to hex$... must be here, otherwise the output is = input (undefined value)
   // get numeric input, convert to integer, format and finally return string output hex$
   //sprintf((char*)PChar(PStrings[HEX_OUT]), "%#04x", int (PInput[WORD_IN]));
   
   // save status of CLK_IN for later use
   PUser[CLK_OLD] = PInput[CLK_IN];
}



/* here whould be the STOP function if needed/used (NOT in this case) */
// void _stdcall CSimStop(double *PInput, double *POutput, double *PUser) { ... }
// Before use it must be declared with DLLEXPORT (see above) as the other functions are.




// called by ProfiLab for DLL configuration dialog
DLLEXPORT void _stdcall CConfigure(double *PUser) {
    MessageBox(NULL, TEXT("Nothing to configure!") /* msg content */ ,
                     TEXT("Configuration:") /* msg title */, MB_OK);
}
